from dataclasses import dataclass
from typing import cast
import uuid

from fastapi import Response
from fastapi.datastructures import QueryParams
from sqlmodel import select

from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.user import User
from openec2.db.instance import Instance
from openec2.utils.array import parse_array_objects, parse_array_plain
from openec2.api.create_tags import CreateTagsResponse


@dataclass
class Tag:
    key: str
    value: str


def create_tags(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
    user: User,
):
    tags: dict[str, str] = {}
    for tag in parse_array_objects("Tag", cast(dict, params)):
        tags[tag["Key"]] = tag["Value"]

    for instance_id in parse_array_plain("ResourceId", cast(dict, params)):
        instance = db.exec(select(Instance).where(Instance.id == instance_id, Instance.owner_id == user.id, Instance.terminated == False)).first()
        if instance is None:
            print(f"Unknown instance {instance_id}")
            continue
        instance.tags = {
            **instance.tags,
            **tags,
        }
        print(instance)

    db.commit()
    return Response(
        CreateTagsResponse(
            requestId=uuid.uuid4().hex,
            return_=True,
        ).to_xml(),
        media_type="application/xml",
    )
