import uuid

from fastapi import Response
from fastapi.datastructures import QueryParams

from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.user import User
from openec2.db.instance import EBSVolume
from openec2.api.create_volume import CreateVolumeResponse


def create_volume(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
    user: User,
):
    availabilityZone = params["AvailabilityZone"]

    volume_id = f"vol-{uuid.uuid4().hex}"
    volume = EBSVolume(
        id=volume_id,
        availability_zone=availabilityZone,
        multi_attach_enabled=params.get("MultiAttachEnabled", "false") == "true",
        owner_id=user.id,
    )
    volume.path(config).mkdir()

    db.add(volume)
    db.commit()

    return Response(
        CreateVolumeResponse(
            requestId=uuid.uuid4().hex,
            volumeId=volume_id,
            availabilityZone=volume.availability_zone,
            multiAttachEnabled=volume.multi_attach_enabled,
        ).to_xml(),
        media_type="application/xml",
    )
