from fastapi import Response, HTTPException
from fastapi.datastructures import QueryParams
from sqlmodel import select

from openec2.db.user import User
from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.vpc import VPC
from openec2.network.vpc import prepare_host_vpc


def create_vpc(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
    user: User,
):
    # TODO: Check if it already exists
    cidr_block = params["CidrBlock"]
    vpcs = db.exec(select(VPC)).all()
    max_interface_num = max(v.bridge_num for v in vpcs) if vpcs else 0

    # Create the VPC
    vpc = VPC(
        bridge_num=max_interface_num + 1,
        cidr=cidr_block,
        owner_id=user.id,
        default=False,
    )
    prepare_host_vpc(vpc)

    db.add(vpc)
    db.commit()
