import uuid

from fastapi import Response, HTTPException
from fastapi.datastructures import QueryParams
from sqlmodel import select

from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.user import User
from openec2.db.instance import Instance
from openec2.api.describe_instance_attribute import DescribeInstanceAttributeResponse, InstanceInitiatedShutdownBehaviour, DisableApiTermination, DisableApiStop


def describe_instance_attribute(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
    user: User,
):
    instance_id = params["InstanceId"]
    attribute = params["Attribute"]
    instance = db.exec(select(Instance).where(Instance.id == instance_id, Instance.owner_id == user.id, Instance.terminated == False)).first()
    if instance is None:
        raise HTTPException(
            status_code=404,
        )

    return Response(
        DescribeInstanceAttributeResponse(
            requestId=uuid.uuid4().hex,
            instanceId=instance.id,
            instanceInitiatedShutdownBehaviour=InstanceInitiatedShutdownBehaviour(
                value="stop",
            ) if attribute == "instanceInitiatedShutdownBehaviour" else None,
            disableApiTermination=DisableApiTermination(
                value="false",
            ) if attribute == "disableApiTermination" else None,
            disableApiStop=DisableApiStop(
                value="false",
            ) if attribute == "disableApiStop" else None,
        ).to_xml(),
        media_type="application/xml",
    )
