import uuid

from sqlmodel import select
from fastapi import Response

from openec2.db import DatabaseDep
from openec2.config import OpenEC2Config
from fastapi.datastructures import QueryParams
from openec2.db.user import User
from openec2.db.instance import EBSVolume
from openec2.api.describe_volumes import DescribeVolumesResponse, VolumeSet, Volume


def describe_volumes(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
    user: User,
):
    volumes = db.exec(select(EBSVolume).where(EBSVolume.owner_id == user.id)).all()
    return Response(
        DescribeVolumesResponse(
            requestId=uuid.uuid4().hex,
            volumeSet=VolumeSet(
                item=[
                    Volume(
                        volumeId=volume.id,
                        multiAttachEnabled=volume.multi_attach_enabled,
                    )
                    for volume in volumes
                ],
            ),
        ).to_xml(),
        media_type="application/xml",
    )
