import uuid

import libvirt
from sqlmodel import select
from fastapi import Response
from fastapi.datastructures import QueryParams

from openec2.libvirt import LibvirtSingleton
from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.user import User
from openec2.db.instance import Instance, EBSVolume
from openec2.api.detach_volume import DetachVolumeResponse
from openec2.utils.libvirt import ebs_volume_to_libvirt_xml


def detach_volume(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
    user: User,
):
    instance_id = params["InstanceId"]
    volume_id = params["VolumeId"]

    # Find the instance
    instance = db.exec(
        select(Instance).where(Instance.id == instance_id, Instance.owner_id == user.id)
    ).first()
    if instance is None:
        return

    # Find the volume
    volume = db.exec(
        select(EBSVolume).where(
            EBSVolume.id == volume_id, EBSVolume.owner_id == user.id
        )
    ).first()
    if volume is None:
        return
    if instance_id not in [i.id for i in volume.instances]:
        return

    # Remove the volume from the instance
    domain = LibvirtSingleton.of().connection.lookupByName(instance_id)
    domain.detachDeviceFlags(
        ebs_volume_to_libvirt_xml(volume, config),
        libvirt.VIR_DOMAIN_DEVICE_MODIFY_LIVE
        if domain.isActive()
        else libvirt.VIR_DOMAIN_DEVICE_MODIFY_CONFIG,
    )

    return Response(
        DetachVolumeResponse(
            requestId=uuid.uuid4().hex,
            volumeId=volume_id,
            instanceId=instance_id,
            status="detached",
        ).to_xml(),
        media_type="application/xml",
    )
