import uuid

from fastapi import Response
from fastapi.datastructures import QueryParams

from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.user import User
from openec2.api.get_caller_identity import GetCallerIdentityResponse, GetCallerIdentityResult, ResponseMetadata


def get_caller_identity(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
    user: User,
):
    return Response(
        GetCallerIdentityResponse(
            result=GetCallerIdentityResult(
                arn=f"arn:aws:iam::{user.id}:user/{user.name}",
                user_id=str(user.id),
                account=str(user.id),
            ),
            metadata=ResponseMetadata(
                request_id=uuid.uuid4().hex,
            ),
        ).to_xml(),
        media_type="application/xml",
    )
