from pydantic_xml import BaseXmlModel, wrapped, element
import libvirt

from openec2.db.instance import Instance
from openec2.api.shared import InstanceState, Tag

class InstanceDescription(
    BaseXmlModel,
    tag="item",
):
    instanceId: str = element()
    imageId: str = element()
    instanceState: InstanceState = element()
    privateIpAddress: str = element()
    tagSet: list[Tag] = wrapped("tagSet", element(tag="item"))


class ReservationSet(BaseXmlModel):
    reservationId: str = element()
    instancesSet: list[InstanceDescription] = wrapped("instancesSet", element(tag="item"))


class DescribeInstancesResponse(
    BaseXmlModel,
    tag="DescribeInstancesResponse",
    nsmap={"": "http://ec2.amazonaws.com/doc/2016-11-15/"},
):
    requestId: str = element()
    reservationSet: list[ReservationSet] = wrapped("reservationSet", element("item"))


def describe_instance_state(domain: libvirt.virDomain) -> InstanceState:
    running = domain.isActive()
    return InstanceState(
        code=16 if running else 80,
        name="running" if running else "stopped",
    )


def describe_instance(
    instance: Instance, domain: libvirt.virDomain
) -> InstanceDescription:
    return InstanceDescription(
        instanceId=instance.id,
        imageId=instance.imageId,
        instanceState=describe_instance_state(domain),
        privateIpAddress=instance.privateIPv4,
        tagSet=[
            Tag(
                key=key,
                value=value,
            )
            for key, value in instance.tags.items()
        ],
    )
