from ipaddress import ip_network

from sqlmodel import SQLModel, Field


class VPC(SQLModel, table=True):
    # ID of the VPC
    id: str = Field(default=None, primary_key=True)

    # Number-suffix of the bridge device
    bridge_num: int

    # Base IPv4
    cidr: str

    # Flag indicating whether this VPC is the user's default VPC
    default: bool

    # Owning user
    owner_id: int = Field(foreign_key="user.id")

    @property
    def virbr(self) -> str:
        return f"virbr{self.bridge_num}"

    @property
    def gateway(self) -> str:
        network = ip_network(self.cidr)
        return f"{network[1]}/{network.prefixlen}"

    @property
    def broadcast(self) -> str:
        network = ip_network(self.cidr)
        return f"{network[255]}"
