from typing import Callable


def parse_array_objects(prefix: str, params: dict) -> list[dict[str, str]]:
    items: dict[str, dict[str, str]] = {}
    for key, value in params.items():
        if not key.startswith(prefix):
            continue

        parts = key.split(".")
        if parts[1] not in items:
            items[parts[1]] = {}
        items[parts[1]][".".join(parts[2:])] = value
    return list(items.values())


def parse_array_plain(prefix: str, params: dict[str, str]) -> list[str]:
    items: dict[str, str] = {}
    for key, value in params.items():
        if not key.startswith(prefix):
            continue

        cleaned_key = key.replace(f"{prefix}.", "")
        if not cleaned_key.isnumeric():
            continue

        items[cleaned_key] = value

    indices = sorted(list(items.keys()))
    return [items[key] for key in indices]


def find[T](l: list[T], pred: Callable[[T], bool]) -> T | None:
    for item in l:
        if pred(item):
            return item
    return None


def parse_tag_specification(params: dict) -> dict[str, str]:
    tags: dict[str, str] = {}
    for spec in parse_array_objects("TagSpecification", params):
        for raw_tag in parse_array_objects("Tag", spec):
            tags[raw_tag["Key"]] = raw_tag["Value"]
    return tags
