from openec2.config import OpenEC2Config
from openec2.db.instance import EBSVolume, Instance


def ebs_volume_to_libvirt_xml(volume: EBSVolume, config: OpenEC2Config) -> str:
    # TODO: Honour the attached device name
    return f"""
        <filesystem type='mount' accessmode='passthrough'>
            <driver type='virtiofs' queue='1024' />
            <source dir='{config.instances.volumes / volume.id}' />
            <target dir='{volume.id}' />
        </filesystem>
    """


def instance_to_libvirt_xml(
    instance: Instance,
    config: OpenEC2Config,
    uuid: str | None = None,
) -> str:
    instance_type = config.instances.types[instance.instanceType]
    ami_path = config.instances.location / instance.id
    memory_backing = (
        """
    <memoryBacking>
        <source type='memfd' />
        <access mode='shared' />
    </memoryBacking>
"""
        if instance.ebs_volumes
        else ""
    )

    volumes = "\n".join(
        ebs_volume_to_libvirt_xml(volume, config) for volume in instance.ebs_volumes
    )

    uuid_element = f"<uuid>{uuid}</uuid>" if uuid is not None else ""
    return f"""<domain type='kvm'>
    {uuid_element}
    <name>{instance.id}</name>
    <memory unit='MiB'>{instance_type.memory}</memory>
    {memory_backing}
    <vcpu placement='static'>{int(instance_type.vcpu)}</vcpu>
    <os>
        <type arch='x86_64'>hvm</type>
        <boot dev='hd' />
        <smbios mode='sysinfo' />
    </os>
    <sysinfo type='smbios'>
        <system>
            <entry name='serial'>ds=nocloud;s=http://192.168.122.1:8000/private/cloudinit/{instance.id}/</entry>
        </system>
    </sysinfo>
    <features>
        <acpi />
        <apic />
        <vmport state='off' />
    </features>
    <clock offset='utc'>
        <timer name='rtc' tickpolicy='catchup'/>
        <timer name='pit' tickpolicy='delay'/>
        <timer name='hpet' present='no'/>
    </clock>
    <pm>
        <suspend-to-mem enabled='no'/>
        <suspend-to-disk enabled='no'/>
    </pm>
    <devices>
        {volumes}
        <disk type='file' device='disk'>
            <driver name='qemu' type='qcow2'/>
            <source file='{ami_path}'/>
            <target dev='vda' bus='virtio'/>
        </disk>
        <rng model="virtio">
            <backend model="random">/dev/urandom</backend>
        </rng>
        <interface type="network">
            <source network="default"/>
            <mac address="{instance.interfaceMac}" />
            <model type="virtio"/>
        </interface>
    </devices>
</domain>
"""
